Index: src/arm/deoptimizer-arm.cc |
diff --git a/src/arm/deoptimizer-arm.cc b/src/arm/deoptimizer-arm.cc |
index 85b62da9e611ddd23bcf8e3f44d77d549dff99d0..6a7c1466a8c3bdd2e09d3b7df6d0dfaef1f3dd88 100644 |
--- a/src/arm/deoptimizer-arm.cc |
+++ b/src/arm/deoptimizer-arm.cc |
@@ -466,13 +466,19 @@ void Deoptimizer::DoCompiledStubFrame(TranslationIterator* iterator, |
// v +-------------------------+ +-------------------------| |
// | COMPILED_STUB marker | | STUB_FAILURE marker | |
// +-------------------------+ +-------------------------+ |
- // | | | stub parameter 1 | |
+ // | | | caller args.length_ | |
// | ... | +-------------------------+ |
- // | | | ... | |
+ // | | | caller args.arguments_ | |
// |-------------------------|<-sp +-------------------------+ |
// | stub parameter n | |
// parameters in registers +-------------------------+<-sp |
// and spilled to stack r0 = number of parameters |
// r1 = failure handler address |
// fp = saved frame |
// cp = JSFunction context |
@@ -483,9 +489,15 @@ void Deoptimizer::DoCompiledStubFrame(TranslationIterator* iterator, |
CodeStubInterfaceDescriptor* descriptor = |
isolate_->code_stub_interface_descriptor(major_key); |
+ // The output frame must have room for all pushed register parameters |
+ // and the standard stack frame slots. |
int output_frame_size = StandardFrameConstants::kFixedFrameSize + |
kPointerSize * descriptor->register_param_count_; |
+ // Include space for an argument object to the callee and optionally |
+ // the space to pass the argument object to the stub failure handler. |
+ output_frame_size += sizeof(Arguments) + kPointerSize; |
+ |
FrameDescription* output_frame = |
new(output_frame_size) FrameDescription(output_frame_size, 0); |
ASSERT(frame_index == 0); |
@@ -497,12 +509,15 @@ void Deoptimizer::DoCompiledStubFrame(TranslationIterator* iterator, |
reinterpret_cast<intptr_t>(notify_failure->entry())); |
Code* trampoline = NULL; |
- StubFailureTrampolineStub().FindCodeInCache(&trampoline, isolate_); |
+ int extra = descriptor->extra_expression_stack_count_; |
+ StubFailureTrampolineStub(extra).FindCodeInCache(&trampoline, isolate_); |
ASSERT(trampoline != NULL); |
output_frame->SetPc(reinterpret_cast<intptr_t>( |
trampoline->instruction_start())); |
unsigned input_frame_size = input_->GetFrameSize(); |
+ intptr_t frame_ptr = input_->GetRegister(fp.code()); |
+ |
// JSFunction continuation |
intptr_t input_frame_offset = input_frame_size - kPointerSize; |
intptr_t output_frame_offset = output_frame_size - kPointerSize; |
@@ -528,6 +543,28 @@ void Deoptimizer::DoCompiledStubFrame(TranslationIterator* iterator, |
Smi::FromInt(StackFrame::STUB_FAILURE_TRAMPOLINE)); |
output_frame->SetFrameSlot(output_frame_offset, value); |
+ int caller_arg_count = 0; |
+ if (descriptor->stack_parameter_count_ != NULL) { |
+ caller_arg_count = |
+ input_->GetRegister(descriptor->stack_parameter_count_->code()); |
+ } |
+ |
+ // Build the Arguments object for the caller's parameters and a pointer to it. |
+ output_frame_offset -= kPointerSize; |
+ value = frame_ptr + StandardFrameConstants::kCallerSPOffset + |
+ (caller_arg_count - 1) * kPointerSize; |
+ output_frame->SetFrameSlot(output_frame_offset, value); |
+ |
+ output_frame->SetFrameSlot(output_frame_offset, value); |
+ output_frame_offset -= kPointerSize; |
+ output_frame->SetFrameSlot(output_frame_offset, caller_arg_count); |
+ |
+ value = frame_ptr - (output_frame_size - output_frame_offset) - |
+ StandardFrameConstants::kMarkerOffset; |
+ output_frame_offset -= kPointerSize; |
+ output_frame->SetFrameSlot(output_frame_offset, value); |
+ |
+ // Copy the register parameters to the failure frame. |
for (int i = 0; i < descriptor->register_param_count_; ++i) { |
output_frame_offset -= kPointerSize; |
DoTranslateCommand(iterator, 0, output_frame_offset); |
@@ -538,14 +575,17 @@ void Deoptimizer::DoCompiledStubFrame(TranslationIterator* iterator, |
output_frame->SetDoubleRegister(i, double_value); |
} |
- value = input_->GetRegister(fp.code()); |
- output_frame->SetRegister(fp.code(), value); |
- output_frame->SetFp(value); |
+ output_frame->SetRegister(fp.code(), frame_ptr); |
+ output_frame->SetFp(frame_ptr); |
ApiFunction function(descriptor->deoptimization_handler_); |
ExternalReference xref(&function, ExternalReference::BUILTIN_CALL, isolate_); |
intptr_t handler = reinterpret_cast<intptr_t>(xref.address()); |
- output_frame->SetRegister(r0.code(), descriptor->register_param_count_); |
+ int params = descriptor->register_param_count_; |
+ if (descriptor->stack_parameter_count_ != NULL) { |
+ params++; |
+ } |
+ output_frame->SetRegister(r0.code(), params); |
output_frame->SetRegister(r1.code(), handler); |
} |
@@ -466,13 +466,19 @@ void Deoptimizer::DoCompiledStubFrame(TranslationIterator* iterator, |
// |-------------------------|<-sp +-------------------------+ |
- // | stub parameter n | |
- // parameters in registers +-------------------------+<-sp |
- // and spilled to stack r0 = number of parameters |
+ // | caller args pointer | |
+ // +-------------------------+ |
+ // | caller stack param 1 | |
+ // parameters in registers +-------------------------+ |
+ // and spilled to stack | .... | |
+ // +-------------------------+ |
+ // | caller stack param n | |
+ // +-------------------------+<-sp |
+ // r0 = number of parameters |
// r1 = failure handler address |
// fp = saved frame |